@@ -340,55 +340,66 @@ def test_create_and_delete_list(karakeep_client: KarakeepAPI):
             print("\nSkipping deletion because list creation failed or ID was not obtained.")
 
 
-def test_create_and_delete_bookmark(karakeep_client: KarakeepAPI):
-    """Test creating a new URL bookmark and then deleting it."""
-    created_bookmark_id = None
-    test_url = "https://en.wikipedia.org/wiki/Example"
+def test_create_and_delete_bookmark(karakeep_client: KarakeepAPI, managed_bookmark: datatypes.Bookmark):
+    """
+    Test verifying a created bookmark (via fixture) and searching for it.
+    The fixture handles creation and deletion.
+    """
+    created_bookmark_id = managed_bookmark.id
+    test_url = managed_bookmark.content.url # Get URL from fixture
+    original_title = managed_bookmark.title # Get title from fixture
+
     try:
-        # 1. Define bookmark payload
-        print(f"\nAttempting to create bookmark for URL: {test_url}")
+        # 1. Bookmark is already created by the 'managed_bookmark' fixture.
+        print(f"\nUsing managed bookmark ID: {created_bookmark_id}, URL: '{test_url}', Title: '{original_title}'")
 
-        # 2. Create the bookmark
-        # Call the method with keyword arguments matching its signature
-        created_bookmark = karakeep_client.create_a_new_bookmark(
-            type="link", url=test_url
-        )
-        assert isinstance(
-            created_bookmark, datatypes.Bookmark
-        ), "Response should be a Bookmark model"
-        assert created_bookmark.content.url == test_url, "Created bookmark URL should match"
-        assert created_bookmark.id, "Created bookmark must have an ID"
-        created_bookmark_id = created_bookmark.id
-        print(f"✓ Successfully created bookmark with ID: {created_bookmark_id}")
-
-        # 3. Verify the bookmark exists by getting it directly
+        # 2. Verify the bookmark exists by getting it directly
         retrieved_bookmark = karakeep_client.get_a_single_bookmark(
             bookmark_id=created_bookmark_id
         )
         assert isinstance(retrieved_bookmark, datatypes.Bookmark)
         assert retrieved_bookmark.id == created_bookmark_id
         assert retrieved_bookmark.content.url == test_url
-        print(f"✓ Successfully retrieved the created bookmark by ID.")
+        assert retrieved_bookmark.title == original_title
+        print(f"✓ Successfully retrieved the managed bookmark by ID.")
 
-        # 4. Search for the created bookmark
-        print(f"\nAttempting to search for bookmark with query: 'wikipedia'")
-        search_query = "Example - Wikipedia"
-        search_results = karakeep_client.search_bookmarks(q=search_query, limit=50)
-        assert isinstance(
-            search_results, datatypes.PaginatedBookmarks
-        ), "Search response should be PaginatedBookmarks model"
-        assert isinstance(
-            search_results.bookmarks, list
-        ), "Search results bookmarks attribute should be a list"
-        assert any(
-            b.id == created_bookmark_id for b in search_results.bookmarks
-        ), f"Created bookmark {created_bookmark_id} not found in search results for '{search_query}'"
-        print(f"✓ Found created bookmark in search results for '{search_query}'.")
 
-        # 4a. Test CLI search equivalent
-        print(f"\n  Running CLI equivalent: search-bookmarks --q '{search_query}' --limit 10")
+        # 3. Search for the created bookmark
+        # Use a search query that is likely to match the fixture's title
+        # The fixture title is "Managed Fixture Bookmark {timestamp}-{random_suffix}"
+        # A simple search for "Managed Fixture Bookmark" should work.
+        # If the title is very dynamic, searching by URL might be more robust if supported,
+        # or by a known part of the title.
+        
+        # waiting a bit for the indexation just in case
+        time.sleep(30)
+
+        search_query_component = original_title.split(" ")[0] + " " + original_title.split(" ")[1]
+        print(f"\nAttempting to search for bookmark with query based on title: '{search_query_component}'. Retrying multiple times because search is nondeterministic.")
+        
+        for trial in range(5):
+            search_results = karakeep_client.search_bookmarks(q=search_query_component, limit=100, include_content=False)
+            assert isinstance(
+                search_results, datatypes.PaginatedBookmarks
+            ), "Search response should be PaginatedBookmarks model"
+            assert isinstance(
+                search_results.bookmarks, list
+            ), "Search results bookmarks attribute should be a list"
+            
+            titles_in_search = [b.title for b in search_results.bookmarks]
+            found_in_search = any(b.id == created_bookmark_id for b in search_results.bookmarks)
+            if found_in_search:
+                break
+            else:
+                time.sleep(3)
+        assert found_in_search, \
+        f"Managed bookmark {created_bookmark_id} (Title: '{original_title}') not found in {trial} different search results for '{search_query_component}'. Titles were: '{titles_in_search}'."
+        print(f"✓ Found managed bookmark in search results for '{search_query_component}'.")
+
+        # 4. Test CLI search equivalent
+        print(f"\n  Running CLI equivalent: search-bookmarks --q '{search_query_component}' --limit 10 --include-content false")
         try:
-            cli_search_command = f"python -m karakeep_python_api search-bookmarks --q '{search_query}' --limit 10"
+            cli_search_command = f"python -m karakeep_python_api search-bookmarks --q '{search_query_component}' --limit 10 --include-content false"
             search_cli_output = subprocess.run(
                 cli_search_command,
                 shell=True,
@@ -396,120 +407,41 @@ def test_create_and_delete_bookmark(karakeep_client: KarakeepAPI):
                 capture_output=True,
                 text=True,
             )
-            # Basic check: Ensure the created ID is somewhere in the output JSON
-            # A more robust check would parse the JSON and verify structure/content
-            assert created_bookmark_id in search_cli_output.stdout, f"Created bookmark ID {created_bookmark_id} not found in CLI search output for '{search_query}'"
+            assert created_bookmark_id in search_cli_output.stdout, \
+                f"Managed bookmark ID {created_bookmark_id} not found in CLI search output for '{search_query_component}'"
             print("✓ CLI search command executed successfully and contained the bookmark ID.")
         except subprocess.CalledProcessError as e:
             print(f"  CLI search command failed with exit code {e.returncode}")
             print(f"  Stdout: {e.stdout}")
             print(f"  Stderr: {e.stderr}")
-            pytest.fail(f"CLI command 'search-bookmarks --q {search_query}' failed: {e}")
+            pytest.fail(f"CLI command 'search-bookmarks --q \"{search_query_component}\"' failed: {e}")
         except Exception as e:
             pytest.fail(f"An unexpected error occurred running the CLI search command: {e}")
 
-
     except (APIError, AuthenticationError) as e:
-        pytest.fail(f"API error during bookmark creation/verification/search: {e}")
+        pytest.fail(f"API error during bookmark verification/search: {e}")
     except Exception as e:
         pytest.fail(
-            f"An unexpected error occurred during bookmark creation/verification/search: {e}"
+            f"An unexpected error occurred during bookmark verification/search: {e}"
         )
-    finally:
-        # 5. Delete the bookmark (ensure cleanup)
-        if created_bookmark_id:
-            print(f"\nAttempting to delete bookmark with ID: {created_bookmark_id}")
-            try:
-                karakeep_client.delete_a_bookmark(bookmark_id=created_bookmark_id)
-                print(f"✓ Successfully deleted bookmark with ID: {created_bookmark_id}")
-
-                # 6. Verify the bookmark is gone by trying to get it (should fail)
-                try:
-                    karakeep_client.get_a_single_bookmark(
-                        bookmark_id=created_bookmark_id
-                    )
-                    pytest.fail(
-                        f"Bookmark with ID {created_bookmark_id} should not exist after deletion, but get_a_single_bookmark succeeded."
-                    )
-                except APIError as e:
-                    assert (
-                        e.status_code == 404
-                    ), f"Expected 404 Not Found when getting deleted bookmark, but got status {e.status_code}"
-                    print(
-                        f"✓ Confirmed bookmark {created_bookmark_id} is deleted via API (received 404)."
-                    )
-
-                # 7. Attempt to delete the same bookmark via CLI (should ideally fail or do nothing)
-                print(f"\n  Attempting CLI deletion for already deleted bookmark ID: {created_bookmark_id}")
-                try:
-                    cli_delete_command = f"python -m karakeep_python_api delete-a-bookmark --bookmark-id {created_bookmark_id}"
-                    subprocess.run(
-                        cli_delete_command,
-                        shell=True,
-                        check=True, # Set to False if CLI errors on 404, True if it exits 0
-                        capture_output=True,
-                        text=True,
-                    )
-                    # If check=True and it succeeds, it means the CLI might not error on 404.
-                    print(f"  ✓ CLI delete command executed (check=True implies it might not error on 404).")
-                except subprocess.CalledProcessError as e:
-                    # If check=True, this block means the CLI command failed, which *might* be expected if it errors on 404.
-                    print(f"  ✓ CLI delete command failed as expected (exit code {e.returncode}), likely because bookmark was already deleted.")
-                    # Optional: Assert specific exit code if known
-                    # assert e.returncode == EXPECTED_EXIT_CODE_FOR_NOT_FOUND
-                except Exception as cli_e:
-                    pytest.fail(f"An unexpected error occurred running the CLI delete command: {cli_e}")
-
-                # 8. Verify again via API that the bookmark is still gone after CLI attempt
-                try:
-                    karakeep_client.get_a_single_bookmark(bookmark_id=created_bookmark_id)
-                    pytest.fail(
-                        f"Bookmark {created_bookmark_id} should *still* not exist after CLI deletion attempt, but get_a_single_bookmark succeeded."
-                    )
-                except APIError as e:
-                    assert (
-                        e.status_code == 404
-                    ), f"Expected 404 Not Found after CLI deletion attempt, but got status {e.status_code}"
-                    print(
-                        f"✓ Confirmed bookmark {created_bookmark_id} is *still* deleted after CLI attempt (received 404)."
-                    )
-
-            except (APIError, AuthenticationError) as e:
-                # Catch errors from the initial Python API deletion or subsequent checks
-                pytest.fail(f"API error during bookmark deletion/verification: {e}")
-            except Exception as e:
-                pytest.fail(f"An unexpected error occurred during bookmark deletion: {e}")
-        else:
-            print(
-                "\nSkipping deletion because bookmark creation failed or ID was not obtained."
-            )
+    # No 'finally' block for deletion needed, as 'managed_bookmark' fixture handles it.
+    # The fixture also handles verification of deletion.
 
 
-def test_update_bookmark_title(karakeep_client: KarakeepAPI):
-    """Test updating a bookmark's title via API and CLI."""
-    created_bookmark_id = None
-    # Generate a unique suffix for the initial title to avoid collisions
-    timestamp = int(time.time())
-    random_suffix = "".join(random.choices(string.ascii_lowercase + string.digits, k=6))
-    original_title = f"Initial Test Title {timestamp}-{random_suffix}"
-    test_url = "https://en.wikipedia.org/wiki/PageForBookmarkUpdateTest" # A distinct URL for this test
+def test_update_bookmark_title(karakeep_client: KarakeepAPI, managed_bookmark: datatypes.Bookmark):
+    """Test updating a bookmark's title via API and CLI, using a managed bookmark."""
+    created_bookmark_id = managed_bookmark.id
+    original_title = managed_bookmark.title # Get the original title from the fixture
     
     target_api_title = "this is a test title"
     target_cli_title = "this is a test title (CLI)"
 
     try:
-        # 1. Create a bookmark with an initial title
-        print(f"\nAttempting to create bookmark for URL: {test_url} with original title: '{original_title}'")
-        created_bookmark = karakeep_client.create_a_new_bookmark(
-            type="link", url=test_url, title=original_title
-        )
-        assert isinstance(created_bookmark, datatypes.Bookmark), "Response should be a Bookmark model"
-        created_bookmark_id = created_bookmark.id
-        assert created_bookmark.id, "Created bookmark must have an ID"
-        assert created_bookmark.title == original_title, f"Initial title '{created_bookmark.title}' does not match expected '{original_title}'"
-        print(f"✓ Successfully created bookmark ID: {created_bookmark_id}, Title: '{created_bookmark.title}'")
+        # The bookmark is already created by the 'managed_bookmark' fixture.
+        # We have its ID in created_bookmark_id and its original title.
+        print(f"\nUsing managed bookmark ID: {created_bookmark_id}, Original Title: '{original_title}'")
 
-        # 2. Update the bookmark's title using the API client
+        # 1. Update the bookmark's title using the API client
         print(f"\nAttempting to update bookmark ID {created_bookmark_id} title to: '{target_api_title}' via API")
         update_payload_api = {"title": target_api_title}
         updated_bookmark_partial = karakeep_client.update_a_bookmark(
@@ -520,7 +452,7 @@ def test_update_bookmark_title(karakeep_client: KarakeepAPI):
             f"Partial response title '{updated_bookmark_partial.get('title')}' does not match target API title '{target_api_title}'"
         print(f"✓ API call to update_a_bookmark successful. Partial response title: '{updated_bookmark_partial.get('title')}'")
 
-        # 3. Verify the API update by fetching the bookmark again
+        # 2. Verify the API update by fetching the bookmark again
         print(f"\nFetching bookmark ID {created_bookmark_id} to verify API title update.")
         retrieved_bookmark_after_api_update = karakeep_client.get_a_single_bookmark(
             bookmark_id=created_bookmark_id
@@ -530,7 +462,7 @@ def test_update_bookmark_title(karakeep_client: KarakeepAPI):
             f"Retrieved bookmark title '{retrieved_bookmark_after_api_update.title}' does not match expected API-updated title '{target_api_title}'"
         print(f"✓ Successfully verified bookmark title updated by API to: '{retrieved_bookmark_after_api_update.title}'")
 
-        # 4. Test CLI equivalent for updating the bookmark's title
+        # 3. Test CLI equivalent for updating the bookmark's title
         print(f"\n  Running CLI equivalent to update title to: '{target_cli_title}'")
         cli_update_payload_json = json.dumps({"title": target_cli_title})
         # Ensure the JSON string is properly quoted for the shell command
@@ -546,7 +478,7 @@ def test_update_bookmark_title(karakeep_client: KarakeepAPI):
             )
             print("✓ CLI update command executed successfully.")
 
-            # 5. Verify CLI update by fetching the bookmark again
+            # 4. Verify CLI update by fetching the bookmark again
             print(f"\nFetching bookmark ID {created_bookmark_id} to verify CLI title update.")
             retrieved_bookmark_after_cli_update = karakeep_client.get_a_single_bookmark(
                 bookmark_id=created_bookmark_id
@@ -569,18 +501,94 @@ def test_update_bookmark_title(karakeep_client: KarakeepAPI):
         pytest.fail(f"API error during bookmark title update test: {e}")
     except Exception as e:
         pytest.fail(f"An unexpected error occurred during bookmark title update test: {e}")
+    # No finally block needed for deletion, as 'managed_bookmark' fixture handles it.
+
+
+def test_tag_lifecycle_on_bookmark(karakeep_client: KarakeepAPI, managed_bookmark: datatypes.Bookmark):
+    """
+    Test attaching a tag to a bookmark, updating the tag, detaching it, and deleting it.
+    Uses the managed_bookmark fixture.
+    """
+    bookmark_id = managed_bookmark.id
+    timestamp = int(time.time())
+    random_chars = "".join(random.choices(string.ascii_lowercase + string.digits, k=6))
+    initial_tag_name = f"test-tag-{timestamp}-{random_chars}"
+    updated_tag_name = f"updated-tag-{timestamp}-{random_chars}"
+    tag_id_to_manage = None
+
+    try:
+        # 1. Attach a new tag by name to the bookmark
+        print(f"\nAttempting to attach tag '{initial_tag_name}' to bookmark {bookmark_id}")
+        attach_payload = {"tags": [{"tagName": initial_tag_name}]}
+        attach_response = karakeep_client.attach_tags_to_a_bookmark(
+            bookmark_id=bookmark_id, tags_data=attach_payload
+        )
+        assert "attached" in attach_response and len(attach_response["attached"]) == 1, \
+            "Failed to attach tag or response format incorrect"
+        tag_id_to_manage = attach_response["attached"][0]
+        assert isinstance(tag_id_to_manage, str), "Attached tag ID should be a string"
+        print(f"✓ Tag '{initial_tag_name}' attached with ID: {tag_id_to_manage}")
+
+        # 2. Update the tag's name
+        print(f"\nAttempting to update tag {tag_id_to_manage} to name '{updated_tag_name}'")
+        update_payload = {"name": updated_tag_name}
+        updated_tag = karakeep_client.update_a_tag(
+            tag_id=tag_id_to_manage, update_data=update_payload
+        )
+        # Do not check the type because karakeep 0.24.1 has a server side bug
+        # assert isinstance(updated_tag, datatypes.Tag1), "Update tag response should be Tag1 model"
+        # assert updated_tag.name == updated_tag_name, "Tag name was not updated as expected"
+        # print(f"✓ Tag {tag_id_to_manage} updated to name '{updated_tag.name}'")
+        assert updated_tag["name"] == updated_tag_name, "Tag name was not updated as expected"
+        print(f"✓ Tag {tag_id_to_manage} updated to name '{updated_tag['name']}'")
+
+        # 3. Verify tag update by getting it directly
+        print(f"\nFetching tag {tag_id_to_manage} to verify its name is '{updated_tag_name}'")
+        retrieved_tag = karakeep_client.get_a_single_tag(tag_id=tag_id_to_manage)
+        assert isinstance(retrieved_tag, datatypes.Tag1), "Get single tag response should be Tag1 model"
+        assert retrieved_tag.name == updated_tag_name, "Retrieved tag name does not match updated name"
+        assert retrieved_tag.id == tag_id_to_manage, "Retrieved tag ID does not match"
+        print(f"✓ Verified tag {tag_id_to_manage} has name '{retrieved_tag.name}'")
+
+        # 4. Detach the tag from the bookmark
+        print(f"\nAttempting to detach tag {tag_id_to_manage} from bookmark {bookmark_id}")
+        detach_payload = {"tags": [{"tagId": tag_id_to_manage}]}
+        detach_response = karakeep_client.detach_tags_from_a_bookmark(
+            bookmark_id=bookmark_id, tags_data=detach_payload
+        )
+        assert "detached" in detach_response and tag_id_to_manage in detach_response["detached"], \
+            "Failed to detach tag or response format incorrect"
+        print(f"✓ Tag {tag_id_to_manage} detached from bookmark {bookmark_id}")
+
+    except (APIError, AuthenticationError) as e:
+        pytest.fail(f"API error during tag lifecycle test: {e}")
+    except Exception as e:
+        pytest.fail(f"An unexpected error occurred during tag lifecycle test: {e}")
     finally:
-        # 6. Delete the bookmark (ensure cleanup)
-        if created_bookmark_id:
-            print(f"\nAttempting to delete bookmark ID: {created_bookmark_id} during cleanup.")
+        # 5. Delete the tag (ensure cleanup even if assertions fail mid-test)
+        if tag_id_to_manage:
+            print(f"\nAttempting to delete tag {tag_id_to_manage} (cleanup)")
             try:
-                karakeep_client.delete_a_bookmark(bookmark_id=created_bookmark_id)
-                print(f"✓ Successfully deleted bookmark ID: {created_bookmark_id}")
+                karakeep_client.delete_a_tag(tag_id=tag_id_to_manage)
+                print(f"✓ Successfully deleted tag {tag_id_to_manage}")
+
+                # 6. Verify the tag is gone by trying to get it (should fail with 404)
+                try:
+                    karakeep_client.get_a_single_tag(tag_id=tag_id_to_manage)
+                    pytest.fail(
+                        f"Tag {tag_id_to_manage} should not exist after deletion, but get_a_single_tag succeeded."
+                    )
+                except APIError as e:
+                    assert e.status_code == 404, \
+                        f"Expected 404 Not Found when getting deleted tag, but got status {e.status_code}"
+                    print(f"✓ Confirmed tag {tag_id_to_manage} is deleted (received 404).")
+            except (APIError, AuthenticationError) as e:
+                # Log error during cleanup but don't let it mask original test failure
+                print(f"  API error during tag deletion (cleanup) for ID {tag_id_to_manage}: {e}")
             except Exception as e:
-                # Log cleanup failure but don't obscure original test failure if any
-                print(f"Error during cleanup: Failed to delete bookmark {created_bookmark_id}: {e}")
-                # Depending on test policy, you might re-raise or just log
-                # pytest.fail(f"Failed to delete bookmark {created_bookmark_id} during cleanup: {e}")
+                print(f"  Unexpected error during tag deletion (cleanup) for ID {tag_id_to_manage}: {e}")
+        else:
+            print("\nSkipping tag deletion (cleanup) because tag_id was not obtained or test failed before creation.")
 
 
 # --- Test User Info/Stats Endpoints ---
